/// <reference types="codemirror/addon/hint/show-hint" />
import type { Editor, EditorChange } from "codemirror";
import type { Disposer, Stream } from "./utils/event-stream";
export interface EditorEventMap {
    changes: [cm: Editor, changes: EditorChange[]];
    cursorActivity: [cm: Editor];
    contextmenu: [cm: Editor, event: MouseEvent];
    "cmw:contextMenuOpened": [cm: Editor];
    "cmw:contextMenuClosed": [cm: Editor];
    blur: [cm: Editor];
    viewportChange: [cm: Editor, from: number, to: number];
    refresh: [cm: Editor];
}
export interface EditorEventTarget<K extends keyof EditorEventMap> {
    on(type: string, handler: (...args: EditorEventMap[K]) => void): void;
    off(type: string, handler: (...args: EditorEventMap[K]) => void): void;
}
export declare const fromEditorEvent: <K extends "blur" | "contextmenu" | "changes" | "cursorActivity" | "cmw:contextMenuOpened" | "cmw:contextMenuClosed" | "viewportChange" | "refresh">(target: EditorEventTarget<K>, type: K) => Stream<EditorEventMap[K]>;
export declare const onEditorEvent: <K extends "blur" | "contextmenu" | "changes" | "cursorActivity" | "cmw:contextMenuOpened" | "cmw:contextMenuClosed" | "viewportChange" | "refresh">(target: EditorEventTarget<K>, type: K, handler: (args: EditorEventMap[K]) => void) => Disposer;
/**
 * Typed wrapper around `CodeMirror.signal`.
 * @param editor - Editor instance
 * @param name - Event name
 * @param args - Event parameters
 */
export declare const signal: <K extends "blur" | "contextmenu" | "changes" | "cursorActivity" | "cmw:contextMenuOpened" | "cmw:contextMenuClosed" | "viewportChange" | "refresh">(editor: Editor, name: K, ...args: EditorEventMap[K]) => void;
/**
 * Internal object for tracking individual elements and their event listeners
 */
export interface MouseLeaveAllListenerItem {
    over: boolean;
    element: HTMLElement;
    onEnter: EventListenerOrEventListenerObject;
    onLeave: EventListenerOrEventListenerObject;
}
/**
 * Special class for listening to enter/leave events on a group of elements. When none of those elements are
 * being hovered over, a callback is triggered.
 */
export declare class MouseLeaveAllListener {
    private listeners;
    private onMouseLeaveAll;
    /**
     * @param onMouseLeaveAll - Called when none of the tracked elements have a mouse over them.
     */
    constructor(onMouseLeaveAll: Function);
    /**
     * Adds an element to the list of tracked elements.
     * This element will have mouseenter and mouseleave events tracked.
     * @param element
     */
    addElement(element: HTMLElement): void;
    /**
     * Removes an element from the list of tracked elements.
     * If the element is found, we also remove the event listeners.
     * @param element
     * @return {boolean} - true if the element was found, false otherwise
     */
    removeElement(element: HTMLElement): boolean;
    /**
     * Removes all tracked elements from this class.
     */
    dispose(): void;
    private checkHoverState;
}
