"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeContextMenu = exports.showContextMenu = void 0;
const events_1 = require("../events");
const states = new WeakMap();
exports.showContextMenu = (cm, x, y, itemGroups) => {
    if (states.has(cm))
        exports.removeContextMenu(cm);
    const menu = document.createElement("div");
    menu.className = "cmw-context-menu";
    menu.style.cssText = [
        "z-index: 10;",
        "position: absolute;",
        "white-space: nowrap;",
        "transition: opacity 200ms;",
    ].join("");
    for (let i = 0; i < itemGroups.length; ++i) {
        const items = itemGroups[i];
        for (const { label, handler } of items) {
            const menuItem = menu.appendChild(document.createElement("div"));
            menuItem.className = "cmw-context-menu-item";
            menuItem.appendChild(document.createTextNode(label));
            if (!handler) {
                menuItem.setAttribute("disabled", "");
            }
            else {
                menuItem.addEventListener("click", (e) => {
                    e.preventDefault();
                    exports.removeContextMenu(cm);
                    handler();
                });
            }
        }
        if (i + 1 < itemGroups.length) {
            const sep = menu.appendChild(document.createElement("hr"));
            sep.className = "cmw-context-menu-item-separator";
        }
    }
    menu.style.opacity = "0";
    document.body.appendChild(menu);
    const menuWidth = menu.offsetWidth + 10;
    const menuHeight = menu.offsetHeight + 10;
    menu.style.left = Math.min(x, window.innerWidth - menuWidth) + "px";
    menu.style.top = Math.min(y, window.innerHeight - menuHeight) + "px";
    menu.style.opacity = "1";
    events_1.signal(cm, "cmw:contextMenuOpened", cm);
    const onMousedownOutside = (e) => {
        if (!(e.target instanceof Node) || !menu.contains(e.target)) {
            exports.removeContextMenu(cm);
            document.removeEventListener("mousedown", onMousedownOutside);
        }
    };
    document.addEventListener("mousedown", onMousedownOutside);
    states.set(cm, { menu });
};
exports.removeContextMenu = (cm) => {
    const state = states.get(cm);
    if (!state)
        return;
    if (state.menu)
        state.menu.remove();
    states.delete(cm);
    events_1.signal(cm, "cmw:contextMenuClosed", cm);
};
//# sourceMappingURL=context-menu.js.map