"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.piped = exports.filter = exports.debouncedBuffer = exports.debounce = exports.skipDuplicates = exports.tap = exports.map = exports.fromDomEvent = void 0;
// Creating Streams
exports.fromDomEvent = (target, type) => {
    return (cb) => {
        target.addEventListener(type, cb);
        return () => {
            target.removeEventListener(type, cb);
        };
    };
};
// Operators
// Transform value
exports.map = (transform) => (stream) => {
    return (cb) => stream((x) => cb(transform(x)));
};
// Perform side effect
exports.tap = (effect) => (stream) => {
    return (cb) => {
        return stream((x) => {
            effect(x);
            cb(x);
        });
    };
};
// stream: ____1____2____2____3
// result: ____1____2_________3
exports.skipDuplicates = (equals = (a, b) => a === b) => (stream) => {
    return (cb) => {
        let prev = null;
        return stream((x) => {
            if (!prev || !equals(prev.value, x)) {
                prev = { value: x };
                cb(x);
            }
        });
    };
};
exports.debounce = (ms) => (stream) => {
    return (cb) => {
        let timeout = null;
        return stream((x) => {
            if (timeout !== null) {
                clearTimeout(timeout);
                timeout = null;
            }
            timeout = setTimeout(() => cb(x), ms);
        });
    };
};
exports.debouncedBuffer = (ms) => (stream) => {
    return (cb) => {
        let timeout = null;
        const buffer = [];
        return stream((x) => {
            if (timeout !== null) {
                clearTimeout(timeout);
                timeout = null;
            }
            buffer.push(x);
            timeout = setTimeout(() => {
                cb(buffer.slice());
                buffer.length = 0;
            }, ms);
        });
    };
};
exports.filter = (test) => (stream) => {
    return (cb) => {
        return stream((x) => {
            if (test(x))
                cb(x);
        });
    };
};
// For returning a Stream piped through operators.
exports.piped = (stream, ...ops) => ops.reduce((v, op) => op(v), stream);
//# sourceMappingURL=event-stream.js.map